//
//  ViewController.swift
//  yumbo
//
//  Created by kunii on 2018/09/24.
//  Copyright © 2018年 Takahiro Kunii. All rights reserved.
//

import UIKit
import ARKit

class ViewController: UIViewController {
    // ARシーンを表示する
    @IBOutlet var sceneView:ARSCNView!
    
    //  油圧ショベル
    var yumbo:Excavator!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // 3D仮想シーン環境
        let scene = SCNScene()
        
        // ARシーンを表示するビューへの仮想シーン環境の登録
        sceneView.scene = scene
        
        //  光源ノードを作り3D仮想空間のルートノードに追加
        let lightNode = SCNNode()
        let light = SCNLight()
        lightNode.light = light
        scene.rootNode.addChildNode(lightNode)
        
        //  環境光の追加
        let ambientLightNode = SCNNode()
        let ambientLight = SCNLight()
        ambientLight.type = .ambient
        ambientLight.color = UIColor(white: 0.5, alpha: 1)
        ambientLightNode.light = ambientLight
        
        //  油圧ショベルの作成と登録
        yumbo = Excavator()
        yumbo.root.position = SCNVector3(x:0, y:-1, z:-3)
        scene.rootNode.addChildNode(yumbo.root)
        
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        // ARセッションの初期設定
        let configuration = ARWorldTrackingConfiguration()
        
        // ARセッション開始
        sceneView.session.run(configuration)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        // セッション停止
        sceneView.session.pause()
    }
    
    ///  操作盤（左）の最後の状態保存用
    private var lastLeft = CGPoint.zero
    
    ///  操作盤（左）の対応　挙動は　https://ja.wikipedia.org/wiki/油圧ショベル#JIS方式 参照
    @IBAction func leftCrossHandler(_ control:CrossControl) {
        if lastLeft.y != control.region.y {
            //  アーム伸ばし(上)・曲げ（下）
            yumbo.moveArm(direction:Float(-control.region.y))
            lastLeft.y = control.region.y
        }
        if lastLeft.x != control.region.x {
            //  左右旋回
            yumbo.moveShoulder(direction:Float(-control.region.x))
            lastLeft.x = control.region.x
        }
    }
    
    ///  操作盤（右）の最後の状態保存用
    private var lastRight = CGPoint.zero
    
    ///  操作盤（右）の対応　挙動は　https://ja.wikipedia.org/wiki/油圧ショベル#JIS方式 参照
    @IBAction func rightCrossHandler(_ control:CrossControl) {
        if lastRight.y != control.region.y {
            //  ブーム上下
            yumbo.moveBoom(direction:Float(control.region.y))
            lastRight.y = control.region.y
        }
        if lastRight.x != control.region.x {
            //  バケット掘削（左）・開放（右）
            yumbo.moveBucket(direction:Float(-control.region.x))
            lastRight.x = control.region.x
        }
    }
    
}

